class Error(Exception):
    def __init__(self, message):
        Exception.__init__(self)
        self.message = message

    def __str__(self):
        return repr(self.message)


class InvalidUsage(Exception):
    def __init__(self, message, status_code = 400):
        Exception.__init__(self)
        self.message     = message
        self.status_code = status_code

    def __str__(self): # pragma: no cover
        return '({}) {}'.format(repr(self.status_code), repr(self.message))

    def to_dict(self): # pragma: no cover
        return {
            'errors': [{
                'status': self.status_code,
                'title': self.message
            }]
        }

from .authentication import http_get_provider



from setuptools import setup
from os import path


here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
        long_description = f.read()


setup(
    name    = 'socialauth',
    version = '0.1.0',

    packages = [
        'socialauth',
        'socialauth.providers',
    ],

    description = 'A framework- and backend-independent social login provider.',
    long_description = long_description,

    url = 'https://github.com/emilyhorsman/socialauth',

    author       = 'Emily Horsman',
    author_email = 'me@emilyhorsman.com',

    license = 'MIT',

    install_requires = [
        'httplib2>=0.9.2',
        'oauth2>=1.9.0.post1',
        'PyJWT>=1.4.0'
    ],

    classifiers = [
        'Development Status :: 4 - Beta',

        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)



import os
import json
from urllib.parse import quote

import httplib2
import jwt
from socialauth import Error
from socialauth import InvalidUsage

class Facebook:
    def __init__(self, request_url, params, token_secret, token_cookie):
        self.request_url  = request_url
        self.params       = params
        self.token_cookie = token_cookie
        self.token_secret = token_secret

        self.status       = False
        self.access_token = None
        self.user_id      = None
        self.user_name    = None

        self.client_id = os.environ.get('FACEBOOK_APP_ID', None)
        if not self.client_id:
            raise Error('No FACEBOOK_APP_ID environment value')

        self.client_secret = os.environ.get('FACEBOOK_APP_SECRET', None)
        if not self.client_secret:
            raise Error('No FACEBOOK_APP_SECRET environment value')

        self.api_version = os.environ.get('FACEBOOK_GRAPH_API_VERSION', 'v2.5')

        if params.get('login') == 'start':
            self.start()
        elif 'code' in params:
            self.finish()
        else:
            raise InvalidUsage('Invalid request')

    def start(self):
        url = 'https://www.facebook.com/dialog/oauth?client_id={}&redirect_uri={}'
        url = url.format(self.client_id, quote(self.request_url))
        self.status = 302
        self.redirect = url

    def finish(self):
        self.get_access_token()
        self.get_user_id()
        self.get_user_name()

        if self.user_id is not None:
            self.status = 200

    def get_access_token(self):
        qs = 'client_id={}&redirect_uri={}&client_secret={}&code={}'
        qs = qs.format(
            self.client_id,
            quote(self.request_url),
            self.client_secret,
            self.params.get('code'))

        url  = 'https://graph.facebook.com/{}/oauth/access_token?{}'
        url  = url.format(self.api_version, qs)

        resp, content = httplib2.Http().request(url, 'GET')
        if resp.status != 200:
            raise Error('{} from Facebook'.format(resp.status))

        res = json.loads(content.decode('utf-8'))
        access_token = res.get('access_token', None)
        if access_token is None:
            raise Error('No access token from Facebook')

        self.access_token = access_token
        return access_token

    def get_user_id(self):
        if self.access_token is None:
            raise Error('Must obtain access token with get_access_token first')

        url = 'https://graph.facebook.com/me?fields=id&access_token={}'
        url = url.format(self.access_token, self.access_token)

        resp, content = httplib2.Http().request(url, 'GET')
        if resp.status != 200:
            raise Error('{} from Facebook'.format(resp.status))

        res = json.loads(content.decode('utf-8'))
        user_id = res.get('id', None)
        if not user_id:
            raise Error('No user ID from Facebook')

        self.user_id = user_id
        return user_id

    def get_user_name(self):
        if self.access_token is None:
            raise Error('Must obtain access token with get_access_token first')

        if self.user_id is None:
            raise Error('Must obtain user ID with get_user_id first')

        url = 'https://graph.facebook.com/{}/{}?access_token={}'
        url = url.format(self.api_version, self.user_id, self.access_token)

        resp, content = httplib2.Http().request(url, 'GET')
        if resp.status != 200:
            return None

        res = json.loads(content.decode('utf-8'))
        self.name = res.get('name', None)
        return self.name



import os
from urllib.parse import parse_qsl

import oauth2
import jwt

from socialauth import Error
from socialauth import InvalidUsage


class Twitter:
    def __init__(self, request_url, params, token_secret, token_cookie):
        self.request_url  = request_url
        self.params       = params
        self.token_cookie = token_cookie
        self.token_secret = token_secret

        self.status             = False
        self.user_id            = None
        self.user_name          = None

        self.set_token_cookie   = None
        self.oauth_verifier     = params.get('oauth_verifier', None)
        self.oauth_token        = params.get('oauth_token', None)
        self.oauth_token_secret = None

        self.consumer_key = os.environ.get('TWITTER_CONSUMER_KEY', None)
        if not self.consumer_key:
            raise Error('No TWITTER_CONSUMER_KEY environment value')

        self.consumer_secret = os.environ.get('TWITTER_CONSUMER_SECRET', None)
        if not self.consumer_secret:
            raise Error('No TWITTER_CONSUMER_SECRET environment value')

        self.consumer = oauth2.Consumer(
            key       = self.consumer_key,
            secret    = self.consumer_secret)

        if params.get('login') == 'start':
            self.start()
        elif 'oauth_verifier' in params and 'oauth_token' in params:
            self.finish()
        else:
            raise InvalidUsage('Invalid request')

    def start(self):
        self.get_initial_oauth_tokens()
        self.get_redirect_with_token()

    def finish(self):
        if self.oauth_verifier is None:
            raise Error('No oauth_verifier')

        if self.oauth_token is None:
            raise Error('No oauth_token')

        self.decode_oauth_token_secret()
        self.get_user_information()

    def decode_oauth_token_secret(self):
        if self.token_cookie is None:
            raise Error('No token cookie given')

        try:
            payload = jwt.decode(self.token_cookie,
                                 self.token_secret,
                                 algorithm = 'HS256')
            if payload['data']['type'] != 'oauth_token_secret':
                raise Error('Token does not have an oauth_token_secret in it')

            self.oauth_token_secret = payload['data']['id']
        except:
            raise Error('Failed to retrieve oauth_token_secret from token')

        return self.oauth_token_secret

    def get_user_information(self):
        token = oauth2.Token(self.oauth_token, self.oauth_token_secret)
        token.set_verifier(self.oauth_verifier)
        client = oauth2.Client(self.consumer, token)

        url = 'https://api.twitter.com/oauth/access_token'
        resp, content = client.request(url, 'POST')
        if resp.status != 200:
            raise Error('{} from Twitter'.format(resp.status))

        provider_user = dict(parse_qsl(content.decode('utf-8')))
        if 'user_id' not in provider_user:
            raise Error('No user_id from Twitter')

        self.status    = 200
        self.user_id   = provider_user.get('user_id')
        self.user_name = provider_user.get('screen_name', None)
        return (self.user_id, self.user_name,)

    def get_initial_oauth_tokens(self):
        url = 'https://api.twitter.com/oauth/request_token'
        client = oauth2.Client(self.consumer)
        resp, content = client.request(url, 'GET')
        if resp.status != 200:
            raise Error('{} from Twitter'.format(resp.status))

        oauth_values = dict(parse_qsl(content.decode('utf-8')))
        self.oauth_token        = oauth_values.get('oauth_token')
        self.oauth_token_secret = oauth_values.get('oauth_token_secret')
        if not self.oauth_token or not self.oauth_token_secret:
            raise Error('No oauth_token or oauth_token_secret from Twitter')

        return (self.oauth_token, self.oauth_token_secret,)

    def get_redirect_with_token(self):
        if self.oauth_token is None:
            raise Error('Must obtain oauth_token with get_initial_oauth_tokens first')

        if self.oauth_token_secret is None:
            raise Error('Must obtain oauth_token_secret with get_initial_oauth_tokens first')

        url = 'https://api.twitter.com/oauth/authenticate?oauth_token={}'
        url = url.format(self.oauth_token)

        # Formatted by JSON API spec (jsonapi.org)
        payload = { 'data': {
            'type': 'oauth_token_secret',
            'id': self.oauth_token_secret
        } }

        self.set_token_cookie = jwt.encode(payload,
                                           self.token_secret,
                                           algorithm = 'HS256')
        self.status   = 302
        self.redirect = url
        return (self.redirect, self.set_token_cookie,)



import jwt

import socialauth.providers
from . import InvalidUsage

def validate_provider(provider):
    return provider in (
        'twitter',
        'facebook',
    )


def http_get_provider(provider,
                      request_url, params, token_secret, token_cookie = None):
    if not validate_provider(provider):
        raise InvalidUsage('Provider not supported')

    klass    = getattr(socialauth.providers, provider.capitalize())
    provider = klass(request_url, params, token_secret, token_cookie)
    if provider.status == 302:
        ret = dict(status = 302, redirect = provider.redirect)
        tc  = getattr(provider, 'set_token_cookie', None)
        if tc is not None:
            ret['set_token_cookie'] = tc

        return ret

    if provider.status == 200 and provider.user_id is not None:
        payload = dict(user_id = provider.user_id)
        token   = jwt.encode(payload, token_secret, algorithm = 'HS256')
        ret     = dict(status = 200, set_token_cookie = token)
        if provider.user_name is not None:
            ret['user_name'] = provider.user_name

        return ret

    raise InvalidUsage('Invalid request') # pragma: no cover
